 /* sorgente: OpMat.c */

/* illustra le modalit di allocazione dinamica, di acquisizione e di
** restituzione, e di rilascio della memoria, per una matrice di interi */

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni e tipi funzionali alle varie operazioni di I/O */
#include <stdio.h>

/* inclusione del file di intestazione della libreria standard
** che contiene definizioni di macro, costanti e dichiarazioni
** di funzioni di interesse generale */
#include <stdlib.h>

/*
** definizione della funzione per il recupero della memoria allocata
** per una matrice di interi
*/
void FreeMatInt(int **Mat, int righe)
  { 
  /* definisce una variabile per l'indice di riga */
  int riga;

  /* rilascia la memoria allocata per ognuna delle righe della matrice */
  for(riga=0; riga < righe; riga++)
    free(Mat[riga]);
    
  /* rilascia la memoria allocata per il vettore di accesso alle righe
  ** della matrice */
  free(Mat);
  }
  
/*
** definizione della funzione per l'allocazione a run-time della memoria
** per una matrice di interi
*/
int **AllMatInt(int righe, int colonne)
  {
  /* definizione di una variabile per l'indirizzo di accesso alla matrice e di una
  ** per lindice di riga */
  int** Mat;
  int riga;

  /* inizializzazione della variabile di accesso alla matrice con l'indirizzo di 
  ** un vettore di indirizzi di variabili intere di dimensione pari al 
  ** numero delle righe */
  Mat = (int **) malloc(righe*sizeof(int *));
  
  /* verifica dell'esito della inizializzazione, se negativo termina con NULL */
  if (Mat == NULL)
    return(NULL);
  
  /* inizializzazione di ogni elemento del vettore con l'indirizzo di un
  ** vettore di variabili intere di dimensione pari al numero delle colonne */
  for(riga=0; riga< righe; riga++)
    {
    Mat[riga] = (int *)malloc(colonne*sizeof(int));
    
    /* verifica dell'esito della inizializzazione, se negativo rilascia la
    ** memoria allocata fino a quel momento e termina con NULL */
    if (Mat[riga] == NULL)
      {
	  FreeMatInt(Mat, riga);
      
      return(NULL);
      }
    };
  
    /* restituzione dell'indirizzo di accesso alla matrice */
    return(Mat);
    }

/*
** definizione della funzione per l'acquisizione del contenuto di una matrice di interi
*/
void AcqMatInt(int **Mat, int dim_righe, int dim_col)
  {
  /* definizione delle variabili per l'indice di riga e quello di colonna */
  int riga, col;
  
  /* scansione delle righe della matrice */
  for (riga = 0; riga < dim_righe; riga = riga+1)
    /* scansione delle righe della matrice */
    for (col = 0; col < dim_col; col = col+1) 
      {
      /* acquisizione dell'elemento corrente della matrice */
      printf("\nMat[%d][%d]: ", riga, col);
      scanf("%d", &(Mat[riga][col]));
      };
  }
  
/*
** definizione della funzione per la restituzione del contenuto di una matrice di interi
*/ 

void ResMatInt(int **Mat, int dim_righe, int dim_col)
  {
  /* definizione delle variabili per l'indice di riga e quello di colonna */
  int riga, col;
  
  /* scansione delle righe della matrice */
  for (riga = 0; riga < dim_righe; riga = riga+1)
    /* scansione delle righe della matrice */
    for (col = 0; col < dim_col; col = col+1)
      {
      /* va a capo se necessario */
      if (col == 0)
        printf("\n");
      
      /* visualizzazione dell'elemento corrente */  
      printf("%4d", Mat[riga][col]);
      }
  printf("\n");
  }
  
/*
** definizione della matrice che calcola la somma degli elementi
** sulla riga di una matrice di interi
*/
int sommaRigaMat (
                 int **matrice,
				 int nro_colonne,
				 int ind_riga
				 )
  {
  /* definizione della variabile per la scansione della riga */
  int curs;
  
  /* definizione e inizializzazione della variabile
  ** per la somma degli elementi sulle righe */
  int somma = 0;
  
  /* scandisco la riga */
  for (curs = 0; curs < nro_colonne; curs++)
    {
    /* aggiungo l'elemento corrente alla somma fin qui ottenuta */
	somma = somma+matrice[ind_riga][curs];
	};
	
  /* restituisce la somma degli elementi sulla riga */
  return(somma);
  }

/*
** definizione della funzione per il prodotto riga x colonna
*/
int rigaxColonna (
                 int **Mat1,
                 int **Mat2,
				 int dim,
				 int riga,
				 int colonna
				 )
  {
  /* definizione della variabile per la scansione della riga
  ** e della colonna */
  int curs;
  
  /* definizione e inizializzazione della variabile per il risultato 
  ** del prodotto riga x colonna */
  int somma = 0;
  
  /* scandisco la riga e la colonna corrispondentemente */
  for(curs = 0; curs < dim ; curs ++)
    /* moltiplico gli elementi correnti sulla riga e sulla colonna
	** e aggiungo il prodotto alla somma fin qui calcolata */
	somma = somma + (Mat1[riga][curs] * Mat2[curs][colonna]);

	/* restituisce il rsultato del prodotto riga x colonna */
	return(somma);
  }

/*
** definizione dell funzione per il rpodotto tra matrice
*/
int **prodMatrici (
				   int **Mat1,
				   int **Mat2,
				   int nroRigheMat1,
				   int nroColMat1,
				   int nroColMat2
				   )
  {
  /* variabile per l'indirizzo di accesso alla matrice risultato */
  int **prodotto;
  
  /* definire le variabili per la scansione della matrice risultato */
  int indRiga;
  int indCol;
  
  /* definisco la matrice risultato */
  prodotto = AllMatInt(nroRigheMat1, nroColMat2);
  
  /* se la definizione della matrice risultato fallisce termina con NULL */
  if (prodotto ==  NULL)
	return(NULL);

  /* scandisce la matrice risultato */
  for (indRiga = 0; indRiga < nroRigheMat1; indRiga++)
	for (indCol = 0; indCol < nroColMat2; indCol++)
      /* assegni all'elemento corrente il risultato
	  ** del prodotto riga x colonna di appartenenza */
	  prodotto[indRiga][indCol] = rigaxColonna (
											   Mat1,
											   Mat2,
											   nroColMat1,
											   indRiga,
											   indCol
											   ); 
	
  /* restituisco l'indirizzo di accesso alla matrice risultato */
  return(prodotto);
  }
  
/* testing delle funzioni */

int main ()
  {
  /* definizione delle variabili per l'indirizzo di accesso alle matrici
  ** che partecipano al prodotto e delle loro dimensioni */
  int** Matrice1;
  int** Matrice2;
  int righeMatrice1, colMatrice1, colMatrice2;
  
  /* acquisizione del numero di righe e di colonne della prima matrice */
  printf("Nro righe Matrice 1? ");
  scanf("%d", &righeMatrice1);
  
  printf("Nro colonne Matrice 1? ");
  scanf("%d", &colMatrice1);
  
  printf("Nro colonne Matrice 2? ");
  scanf("%d", &colMatrice2);
  
  /* definisce le matrici che partecipano al prodotto */
  Matrice1 = AllMatInt(righeMatrice1, colMatrice1);
  Matrice2 = AllMatInt(colMatrice1, colMatrice2);

  /* acquisisce, e successivamente restituisce, il contenuto delle
  ** matrici che partecipano al prodotto */
  AcqMatInt(Matrice1, righeMatrice1, colMatrice1);
  ResMatInt(Matrice1, righeMatrice1, colMatrice1);
  
  AcqMatInt(Matrice2, colMatrice1, colMatrice2);
  ResMatInt(Matrice2, colMatrice1, colMatrice2);
  
  /* calcolo il prodotto tra le matrici e lo visualizzo */
	
  ResMatInt (
            prodMatrici (
				        Matrice1,
				        Matrice2,
				        righeMatrice1,
				        colMatrice1,
				        colMatrice2
				        ),
			righeMatrice1,
			colMatrice2
			);
  
  /* termina con 1 */
  return(1);
  } 
